@@ -12,6 +12,7 @@
 #ifndef _SESSION_H
 #define _SESSION_H
 
+#include "config.h"
 #include <time.h>
 #ifdef ENABLE_TLS
 #include "tls.h"
@@ -142,6 +143,10 @@ typedef struct type_session {
 	mbedtls_ssl_context rproxy_ssl;
 #endif
 #endif
+
+#ifdef ENABLE_HTTP2
+	bool            use_http2;
+#endif
 } t_session;
 
 void init_session(t_session *session);
