@@ -564,7 +564,11 @@ int execute_cgi(t_session *session) {
 
 	if ((wrap_cgi == false) && (session->cgi_type != fastcgi)) {
 		check_file_exists = true;
-	} else if ((session->cgi_type == fastcgi) && session->fcgi_server->localhost && session->host->show_index) {
+	} else if ((session->cgi_type == fastcgi) && session->fcgi_server->localhost
+#if defined(ENABLE_XSLT) || defined(ENABLE_MONITOR)
+		&& session->host->show_index
+#endif
+		) {
 		check_file_exists = true;
 	} else {
 		check_file_exists = false;
